# SmallOverlap GAP Package
# Copyright (C) 2007-2008 Mark Kambites, University of Manchester
#
# This package contains code for manipulating semigroups and monoids with
# presentations satisfying small overlap conditions of the kind
# introduced by Remmers.  The algorithms used are described in the paper
# M. Kambites, "Small overlap monoids: the word problem", arXiv ref.
#
# This file contains a few general extra routines for manipulating fp
# semigroups and monoids, which may be useful even to the user not
# interested in small overlap semigroups and monoids

#############################################################################
# Since much of our code works uniformly for finitely presented monoids or
# finitely presented semigroups, it is helpful to have a filter for the
# disjunction of IsFpMonoid and IsFpSemigroup, and of the corresponding
# element filters.

InstallMethod(IsFpMonoidOrSemigroup, "for everything", [IsObject], 
   function(S)
   return IsFpMonoid(S) or IsFpSemigroup(S);
end);

InstallMethod(IsElementOfFpMonoidOrSemigroup, "for everything", [IsObject], 
   function(S)
   return IsElementOfFpMonoid(S) or IsElementOfFpSemigroup(S);
end);


#############################################################################
# Attribute RelationWords of a finitely presented semigroup or monoid is a
# list of the relation words in the defining presentation in their letter
# representations; they are supplied as a single flat list, but the order
# is such that relations can be recovered by considering adjacent pairs.

InstallMethod(RelationWords,
              "for finitely presented monoids and semigroups", 
              [IsFpMonoidOrSemigroup], 
              function(S)
   return List(Flat(RelationsOfFpMonoid(S)), LetterRepAssocWord);
end);

#############################################################################
# Attribute RelationWordLengths is just a list of the lengths of the relation 
# words, in an order corresponding to the attribute RelationWords(). Stored
# to avoid repeated calculation, but I'm not really sure if this is worth
# doing.

InstallMethod(RelationWordLengths,
              "for finitely presented monoids and semigroups",
              [IsFpMonoidOrSemigroup], 
              function(S)
       return List(RelationWords(S), Length);
   end
);

#############################################################################
# The attribute MaxCommonSuffixLengthOfRelationWords is a two dimensional
# array x such that x[i][j] is the length of the longest common suffix of
# relation word i and relation word j. This could be implemented more
# efficiently.

InstallMethod(MaxCommonSuffixLengthOfRelationWords,
              "for finitely presented monoids and semigroups",
              [IsFpMonoidOrSemigroup], 
   function(S)
       local relwords, n;
       relwords := RelationWords(S);
       n := Length(relwords);
       return List([1..n], x ->
                 List([1..n],
                      y -> MaxCommonSuffixLength(relwords[x], relwords[y])));
   end
);
